install.packages("maptools")
install.packages("tripack")
install.packages("spdep") #contiene funzioni per l'analisi spaziale
install.packages("moments")
install.packages("lmtest")
install.packages("nortest")
install.packages("car")
install.packages("spatialreg") #contiene le funzioni per stimare i modelli spaziali
install.packages("leaflet")
install.packages("RColorBrewer") #creazioni di palette di colori per grafici
install.packages("spData")
install.packages("ggplot2")
install.packages("stargazer") #contiene la funzione stargazer per visualizzare gli output dei modelli di regressione
install.packages("sf")         #importazione file
install.packages("rcompanion") #contiene funzioni aggiuntive per analisi statistiche e grafiche es: plotNormalHistogram
install.packages("mapview")    #creazione mappe interattive

library(tripack)
library(spdep)
library(moments)
library(lmtest)
library(nortest)
library(car)
library(spatialreg)
library(leaflet)
library(RColorBrewer)
library(spData)
library(ggplot2)
library(stargazer)
library(readxl)
library(sf)
library(rcompanion)
library(mapview)

####SPATIAL ANALYSIS####
setwd("C:/Users/User/Desktop/progetto")
Premi_assicurativi <- read_excel("progetto.xls")
path=file.choose()
shp=st_read(path) #la funzione st_read importa lo shapefile sottoforma di dataset
dataset=merge(shp,Premi_assicurativi, by='DEN_UTS')

str(dataset)
View(dataset)

summary(dataset$"Premi assicurativi medi")
boxplot(dataset$"Premi assicurativi medi")
hist(dataset$`Premi assicurativi medi`,xlab = "Premi levels",main="Livelli di Premi - frequenze")
attach(dataset)
hist(`Premi assicurativi medi`,probability = T,xlab = "Premi levels",    main = "Livelli di Premi - densit")
plotNormalHistogram(`Premi assicurativi medi`,main="Livelli di Premi - frequenze") 

plot(st_geometry(dataset),col='grey')

mapview(dataset)
mapview(dataset, zcol= 'Premi assicurativi medi')
palette <- brewer.pal(5, "OrRd")
mapview(dataset, zcol='Premi assicurativi medi', layer.name='Livelli di Premi',at=quantile(`Premi assicurativi medi`), col.regions = palette)

#nb queen
mapview(dataset)
list.queen<-poly2nb(dataset, queen=TRUE) #poly2nb crea una lista dei vicini
list.queen
summary(list.queen)
coords<-st_centroid(st_geometry(dataset),of_largest_polygon = TRUE) #la funzione definisce il centroide di ciascun poligono
plot(st_geometry(dataset),col='lightgrey')
plot(list.queen,coords=coords,add=T,col='blue',lwd=2)
#nb rock
list.rook<-poly2nb(dataset, queen=F)
summary(list.rook)
plot(st_geometry(dataset),col='lightgrey')
plot(list.rook,coords=coords,add=T,col='red',lwd=2)

#listwqueen
listw<-nb2listw(list.queen, style="B", zero.policy=TRUE)
View(listw)
#matqueen
W<-nb2mat(list.queen, style="B", zero.policy=TRUE)
View(W)

#distance
coords<-st_centroid(st_geometry(dataset),of_largest_polygon = TRUE)
W_dist <- dnearneigh(coords,0,80000)
W_dist
plot(st_geometry(dataset),col='lightgrey')
plot(W_dist,coords=coords,add=T,col='green',lwd=2)

#confronti
par(mfrow = c(1, 3))
plot(st_geometry(dataset),col='lightgrey',main="QUEEN CRITERION" )
plot(list.queen,coords=coords,add=T,col='blue',lwd=2)
plot(st_geometry(dataset),col='lightgrey',main="ROOK CRITERION" )
plot(list.rook,coords=coords,add=T,col='red',lwd=2)
plot(st_geometry(dataset),col='lightgrey',main="DISTANCE CRITERION" )
plot(W_dist,coords=coords,add=T,col='green',lwd=2)
dev.off()

###########################################
#Moran's I
listw<-nb2listw(list.queen, style="W", zero.policy=TRUE)
moran(`Premi assicurativi medi`,listw=listw, length(`Premi assicurativi medi`),Szero(listw), zero.policy=T)
moran.test(`Premi assicurativi medi`, listw, zero.policy=T)
#plot
moran.plot(`Premi assicurativi medi`,listw,zero.policy=T, pch=20, col="blue")
mp<-moran.plot(`Premi assicurativi medi`,listw, zero.policy=T, pch=20, col="blue")
summary(mp)
#plot valori standardizzati
S_PREMI<- scale(`Premi assicurativi medi`)
moran.plot(as.vector(S_PREMI),listw,zero.policy = T,pch=20,col="blue")
W_S_SPREMI<- scale(lag.listw(listw, `Premi assicurativi medi`))
plot(x = S_PREMI, y = W_S_SPREMI, main = "Moran Scatterplot I = 0.5")
abline(h = 0, v = 0,lty=2,col="blue")
abline(lm(W_S_SPREMI~S_PREMI), lty=1, lwd=2, col="red")

#Geary C
geary.test(`Premi assicurativi medi`,listw,zero.policy = T)

#local Moran's I - LISA
library(rgeoda)
premi_ass = dataset["Premi assicurativi medi"]
listw1=queen_weights(dataset)
summary(listw1)
lisa <- local_moran(listw1, premi_ass)
lisa

lms <- lisa_values(gda_lisa = lisa) #per ottenere i valori del Moran locale
lms

pvals <- lisa_pvalues(lisa) #per otterere i valori p di significativit del Moran locale
pvals

lisa_colors <- lisa_colors(lisa)
lisa_colors
#eeeeee corrisponde a "lightgray"
#FF0000 corrisponde a "red"
#0000FF corrisponde a "blue"
#a7adf9 potrebbe essere descritto come una tonalit di blu pallido o lilla chiaro.
#f4ada8 potrebbe essere descritto come una tonalit di rosa pallido o salmone chiaro.
#464646 corrisponde a "dimgray" o "gray30" (una tonalit di grigio scuro).
#999999 corrisponde a "gray60" (una tonalit di grigio medio).

lisa_labels <- lisa_labels(lisa)
lisa_labels

lisa_clusters <- lisa_clusters(lisa)
lisa_clusters

plot(st_geometry(dataset), 
     col=sapply(lisa_clusters, function(x){return(lisa_colors[[x+1]])}), 
     border = "#333333", lwd=0.2)
title(main = "Local Moran Map of Premi_ass")
legend('bottomleft', legend = lisa_labels, fill = lisa_colors, border = "#eeeeee")

pvals <- lisa_pvalues(lisa)
p_labels <- c("Not significant", "p <= 0.05", "p <= 0.01", "p <= 0.001")
p_colors <- c("#eeeeee", "#84f576", "#53c53c", "#348124")
plot(st_geometry(dataset), 
     col=sapply(pvals, function(x){
       if (x <= 0.001) return(p_colors[4])
       else if (x <= 0.01) return(p_colors[3])
       else if (x <= 0.05) return (p_colors[2])
       else return(p_colors[1])
     }), 
     border = "#333333", lwd=0.2)
title(main = "Local Moran Map of Premi_ass")
legend('bottomleft', legend = p_labels, fill = p_colors, border = "#eeeeee")

###MODELLO OLS###
chi.ols <- lm(`Premi assicurativi medi` ~ Pop_res+Tot_inc+Tot_ve+Tot_mor+Tot_fer+Tas_m+cons_alc, data = dataset)
summary(chi.ols)

###BOX-COX
library(MASS)
intervallo_lambda <- seq(-20, 10, by = 0.01)
bc <- boxcox(chi.ols, lambda = intervallo_lambda)
lambda <- bc$x[which.max(bc$y)]
lambda
hist(1/(`Premi assicurativi medi`)^2)
skewness(1/(`Premi assicurativi medi`)^2)
skewness(`Premi assicurativi medi`)

###TRASFORMAZIONI LOGARITMICHE###
#Modello log-lin
Log_premi <- log(`Premi assicurativi medi`)
log_lin<-lm(Log_premi~ Pop_res+Tot_inc+Tot_ve+Tot_mor+Tot_fer+Tas_m+cons_alc, data=dataset)
summary(log_lin)

#Modello lin-log
Log_pop<-log(Pop_res)
Log_m<-log(Tas_m)
Log_ve<-log(Tot_ve)
Log_inc<-log(Tot_inc)
Log_alc<-log(cons_alc)
Log_mor<-log(Tot_mor)
Log_fer<-log(Tot_fer)
lin_log<-lm(`Premi assicurativi medi`~Log_pop+Log_m+Log_ve+Log_inc+Log_alc+Log_mor+Log_fer, data=dataset)
summary(lin_log)

#Modello log-log
log_log<-lm(Log_premi~Log_pop+Log_m+Log_ve+Log_inc+Log_alc+Log_mor+Log_fer, data=dataset)
summary(log_log)

#CONCLUDIAMO CHE IL MODELLO OLS  IL MIGLIORE. LE TRASFORMAZIONI, IN QUESTO CASO NON RESTITUSCONO UN MODELLO MIGLIORE

chi.ols1 <- lm(1/(`Premi assicurativi medi`)^2 ~ Pop_res+Tot_inc+Tot_ve+Tot_mor+Tot_fer+Tas_m+cons_alc, data = dataset)
summary(chi.ols1)

###STEPWISE###
stepwise=step(chi.ols, directory="both")
ols.new <- lm(`Premi assicurativi medi`~ Tas_m+Tot_ve+Pop_res+Tot_inc+cons_alc, data=dataset)
summary(ols.new)

ols<-lm(1/(`Premi assicurativi medi`)^2~Pop_res+Tot_ve+Tot_inc+cons_alc+Tas_m, data=dataset)
summary(ols)

#LM test
moran.lm<-lm.morantest(ols.new, listw, alternative="two.sided") #verifica la presenza di autocorrelazione spaziale nei residui del modello lineare stimato
moran.lm
LM_tests<-lm.LMtests(ols.new,listw,test="all",zero.policy = T) 
LM_tests

#OLS
ols.new <- lm(`Premi assicurativi medi`~ Tas_m+Tot_ve+Pop_res+Tot_inc+cons_alc, data=dataset)
summary(ols.new)
AIC(ols.new)

#SAR 
sar.chi<-lagsarlm(`Premi assicurativi medi`~ Tas_m+Tot_ve+Pop_res+Tot_inc+cons_alc, data=dataset, listw,tol.solve = 6.2817e-17)
summary(sar.chi)

#SEM
sem.chi<-errorsarlm(`Premi assicurativi medi`~ Tas_m+Tot_ve+Pop_res+Tot_inc+cons_alc, data=dataset, listw)
summary(sem.chi,Nagelkerke=TRUE)
AIC(sem.chi)

stargazer(ols.new, sar.chi, sem.chi,type="text",title = "OLS - SAR - SEM RESULTS")

#SAC
sac.chi<-sacsarlm(`Premi assicurativi medi`~ Tas_m+Tot_ve+Pop_res+Tot_inc+cons_alc, data=dataset, listw, method="LU")
summary(sac.chi,Nagelkerke=TRUE)

#SDM
sdm.chi<-lagsarlm(`Premi assicurativi medi`~ Tas_m+Tot_ve+Pop_res+Tot_inc+cons_alc, data=dataset, listw, type = "mixed", tol.solve = 1.10e-20)
summary(sdm.chi,Nagelkerke=TRUE)

LR.Sarlm(sdm.chi,sem.chi)   #SDM vs SEM
LR.Sarlm(sdm.chi,sar.chi)   #SDM vs SAR

#SDEM
sdem.chi<-errorsarlm(`Premi assicurativi medi`~ Tas_m+Tot_ve+Pop_res+Tot_inc+cons_alc, data=dataset, listw, etype="mixed")
summary(sdem.chi,Nagelkerke=TRUE)
AIC(sdem.chi)

#SLX
slx.chi<-lmSLX(`Premi assicurativi medi`~ Tas_m+Tot_ve+Pop_res+Tot_inc+cons_alc, data=dataset, listw)
summary(slx.chi)
class(slx.chi)

slx.chi2<-lm(`Premi assicurativi medi`~ Tas_m+Tot_ve+Pop_res+Tot_inc+cons_alc +lag.listw(listw,Tas_m)+lag.listw(listw,Tot_ve)+lag.listw(listw,Pop_res)+lag.listw(listw,Tot_inc)+lag.listw(listw,cons_alc), data=dataset)
summary(slx.chi2)
class(slx.chi2)
AIC(slx.chi)
AIC(slx.chi2)

#GNS / Manski Model - General Nesting Spatial
gns.chi<-sacsarlm(`Premi assicurativi medi`~ Tas_m+Tot_ve+Pop_res+Tot_inc+cons_alc, data=dataset, listw,type = "mixed")
summary(gns.chi,Nagelkerke=TRUE)

#Effetti diretti e indiretti  SLX e SDEM 
impacts_slx<-impacts(slx.chi, listw)  
impacts_slx
summary(impacts_slx)
slx.chi$coefficients[-1]

impacts_sdem<-impacts(sdem.chi, listw)
impacts_sdem
summary(impacts_sdem)
sdem.chi$coefficients[-1]

#Effetti diretti e indiretti SAR , SAC e SDM
W2<-as(listw, 'CsparseMatrix')
trMat<-trW(W2,type="mult")

impacts_sar<-impacts(sar.chi,tr=trMat,R=100)
impacts_sar
summary(impacts_sar, zstats=TRUE, short=TRUE)

impacts_sac<-impacts(sac.chi,tr=trMat,R=100)
impacts_sac
summary(impacts_sac, zstats=TRUE, short=TRUE)

impacts_sdm<-impacts(sdm.chi,tr=trMat,R=100)
impacts_sdm
summary(impacts_sdm, zstats=TRUE, short=TRUE)

